@@ -161,12 +161,12 @@ class BadJSON(Exception): pass
 
 def json_deserialize(data):
     if data is None:
-        raise BadJSON(path)
+        raise BadJSON()
 
     try:
         obj = json.loads(data)
     except ValueError:
-        raise BadJSON(path)
+        raise BadJSON()
 
     return obj
 
@@ -546,7 +546,8 @@ class Shell(AugumentedCmd):
         """
         seen = set()
 
-        for path in self._zk.fast_tree(params.path):
+        # we don't want to recurse once there's a child matching, hence exclude_recurse=
+        for path in self._zk.fast_tree(params.path, exclude_recurse=params.pattern):
             parent, child = split(path)
 
             if parent in seen:
@@ -564,6 +565,61 @@ class Shell(AugumentedCmd):
 
     complete_child_matches = _complete_path
 
+    @connected
+    @ensure_params(
+        Optional("path"),
+        IntegerOptional("top", 0)
+    )
+    @check_paths_exists("path")
+    def do_summary(self, params):
+        """
+        Prints a summary of the children for the given [path] (or the current one if none is given)
+
+        summary [path] [top]
+
+        Example:
+
+        summary /services/registrations
+        Created                    Last modified               Owner                Name
+        Thu Oct 11 09:14:39 2014   Thu Oct 11 09:14:39 2014     -                   bar
+        Thu Oct 16 18:54:39 2014   Thu Oct 16 18:54:39 2014     -                   foo
+        Thu Oct 12 10:04:01 2014   Thu Oct 12 10:04:01 2014     0x14911e869aa0dc1   member_0000001
+
+        The results are sorted by name. The top parameter decides the number of results to be
+        displayed.
+
+        """
+
+        self.show_output("%s%s%s%s",
+                         "Created".ljust(32),
+                         "Last modified".ljust(32),
+                         "Owner".ljust(23),
+                         "Name")
+
+        results = sorted(self._zk.stat_map(params.path))
+
+        # what slice do we want?
+        if params.top == 0:
+            start, end = 0, len(results)
+        elif params.top > 0:
+            start, end = 0, params.top if params.top < len(results) else len(results)
+        else:
+            start = len(results) + params.top if abs(params.top) < len(results) else 0
+            end = len(results)
+
+        for i in range(start, end):
+            path, stat = results[i]
+
+            self.show_output(
+                "%s%s%s%s",
+                time.ctime(stat.created).ljust(32),
+                time.ctime(stat.last_modified).ljust(32),
+                ("0x%x" % stat.ephemeralOwner).ljust(23),
+                path[len(params.path) + 1:]
+            )
+
+    complete_summary = _complete_path
+
     @connected
     @ensure_params(Optional("path"), Required("match"))
     @check_paths_exists("path")
@@ -1134,14 +1190,15 @@ child_watches=%s"""
         Required("keys"),
         IntegerOptional("top", 0),
         IntegerOptional("minfreq", 1),
-        BooleanOptional("reverse", default=True)
+        BooleanOptional("reverse", default=True),
+        BooleanOptional("report_errors", default=False)
     )
     @check_paths_exists("path")
     def do_json_count_values(self, params):
         """
         Counts the frequency of values associated with <keys>, for all JSON dicts stored in <path>'s children
 
-        json_count_values <path> <keys> [top] [minfreq] [reverse]
+        json_count_values <path> <keys> [top] [minfreq] [reverse] [report_errors]
 
         Example:
 
@@ -1154,7 +1211,8 @@ child_watches=%s"""
         ...
 
         By default, all values are shown (top = 0) regardless of their frequency (minfreq = 1).
-        They are sorted by frequency in descendant order (reverse = true).
+        They are sorted by frequency in descendant order (reverse = true). Errors like bad JSON
+        or missing keys are not reported by default (report_errors = false).
 
         """
         try:
@@ -1171,23 +1229,31 @@ child_watches=%s"""
                 value = Keys.value(json_deserialize(data), params.keys)
                 values[value] += 1
             except BadJSON as ex:
-                self.show_output("Path %s has bad JSON.", path)
+                if params.report_errors:
+                    self.show_output("Path %s has bad JSON.", path)
             except Keys.Missing as ex:
-                self.show_output("Path %s is missing key %s.", path, ex)
+                if params.report_errors:
+                    self.show_output("Path %s is missing key %s.", path, ex)
 
         results = sorted(values.items(), key=lambda item: item[1], reverse=params.reverse)
+        results = [r for r in results if r[1] >= params.minfreq]
 
-        i = 0
-        for value, frequency in results:
-            if frequency < params.minfreq:
-                continue
+        # what slice do we want?
+        if params.top == 0:
+            start, end = 0, len(results)
+        elif params.top > 0:
+            start, end = 0, params.top if params.top < len(results) else len(results)
+        else:
+            start = len(results) + params.top if abs(params.top) < len(results) else 0
+            end = len(results)
 
+        for i in range(start, end):
+            value, frequency = results[i]
             self.show_output("%s = %d", value, frequency)
 
-            if params.top > 0 and i > params.top:
-                break
-
-            i += 1
+        # if no results were found we call it a failure (i.e.: exit(1) from --run-once)
+        if len(results) == 0:
+            return False
 
     complete_json_count_values = _complete_path
 
